@@ -27,6 +27,15 @@ int add(int a, int b) {
 	return a + b;
 }
 
+void dummy(int a, int b) {
+	Rcpp::Rcout << "dummy called" << std::endl;
+}
+
+// [[Rcpp::export]]
+Rcpp::InternalFunction getDummy() {
+	return Rcpp::InternalFunction( &dummy );
+}
+
 
 // [[Rcpp::export]]
 Rcpp::InternalFunction getAdd() {
