@@ -1,8 +1,7 @@
-// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; tab-width: 8 -*-
-//
+
 // sugar.cpp: Rcpp R/C++ interface class library -- sugar unit tests
 //
-// Copyright (C) 2012 - 2015  Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2012 - 2022  Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -23,7 +22,12 @@
 using namespace Rcpp ;
 
 template <typename T>
+
+#if __cplusplus < 201103L
 class square : public std::unary_function<T,T> {
+#else
+class square : public std::function<T(T)> {
+#endif
 public:
 	T operator()( T t) const { return t*t ; }
 } ;
